# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tjson']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'tj-json',
    'version': '1.0.1',
    'description': 'Accesss utility for JSON-shaped Python objects',
    'long_description': '# `tjson` &mdash; An access utility for typed "JSON-shaped" Python objects\n\nHave you had to deal with deeply nested JSON documents in Python, only to be greeted by `KeyError`? What about only to run across weird type issues, and mysterious bugs? Have you wished that you could easily get type hinting on values that you pull out of your JSON documents? `tjson` is here to help.\n\n## Installing\n\n`tjson` has no dependencies outside the standard Python library. Simply install it with Pip or your favorite dependency manager.\n\n    pip install tj_json\n\n> `tjson` is taken on PyPI by a dead project, and I haven\'t bothered to take over the name.\n\n## Usage\n\n`tjson`\'s API  is designed to be intuitive and simple.\n\n```py\n>>> from tjson.tjson import TJ\n```\n\n`TJ` is a generic wrapper for JSON values. Simply give it your "JSON-shaped" Python values to get started.\n\n> **Note: `tjson` is an access utility, not a parser/deserializer.** To parse your JSON, you must still use a parser such as the standard library\'s `json`.\n\n```py\n>>> mydata = TJ({\'who\': \'the quick brown fox\', \'what\': \'jumped\', \'where\': [\'over\', \'the lazy dog\'], \'timestamp\': 12345.123})\n```\n\n### Traversal and access\n\nTo traverse `TJ`, simply use Python\'s item accessor `[]`. All usees of it return a new instance of `TJ` pointing to the new value. To retrieve the value wrapped by `TJ`, just check its `.value`.\n\n```py\n>>> mydata[\'where\'][1]\n<tjson.tjson.TJ object at 0x7f02148e2f20>\n>>> mydata[\'where\'][1].value\n\'the lazy dog\'\n```\n\n`TJ` objects also store the path that they followed to reach them, and that can be queried using `.path`.\n\n```py\n>>> mydata[\'where\'][1].path\n\'.where[1]\'\n```\n\n### When things go wrong: Warnings\n\nWhen using regular Python `dict`/`list` strucures, trying to access a bad key or index results in exceptions. This makes you need either over-broad or over-nitpicky exception handling, and makes code smell. `tjson` instead relegates problems accessing our JSON documents as warnings, taking advantage of Python\'s smart [`warnings` feature set](https://docs.python.org/3/library/warnings.html).\n\nWhen access to the underlying object fails, a `TJ` instance is created anyway, just wrapping the value `None`. Additionally, an appropriate warning is fired off using `warnings.warn`, and the warnings are collected in the `.warnings` property of the `TJ`.\n\n```py\n>>> print(*mydata[\'who\'][\'entities\'][123][\'name\'].warnings, sep="\\n")\n\n<stdin>:1: InvalidKeyWarning: Tried to access str key \'entities\' of non-object at path `.who`\n\nTried to access str key \'entities\' of non-object at path `.who`\nCannot access int index 123 of non-array at path .who.entities\nTried to access str key \'name\' of non-object at path `.who.entities[123]`\n```\n\n> Only the first warning in a chain of warnings is actually sent to `warnings.warn`. This is to avoid flooding if an error happens early in a complex access chain.\n\nIf you want the bad accesses to raise warnings instead, you can use the `warnings` API to enable the specific class:\n\n```py\n>>> import warnings\n>>> from tjson.errors import InvalidKeyWarning\n>>> warnings.simplefilter(\'error\', InvalidKeyWarning)\n>>> mydata[\'who\'][\'entities\'][123][\'name\']\nTraceback (most recent call last):\n  File "<stdin>", line 1, in <module>\n  File "/home/fsufitch/code/tjson/tjson/tjson.py", line 30, in __getitem__\n    return TJ(None, next_path, _amend_warns(self.warnings, InvalidKeyWarning(f"Tried to access str key {repr(key)} of non-object at path `{self.path}`"), 2))\n  File "/home/fsufitch/code/tjson/tjson/tjson.py", line 123, in _amend_warns\n    warn(warning, stacklevel=stacklevel + 1)\ntjson.errors.InvalidKeyWarning: Tried to access str key \'entities\' of non-object at path `.who`\n```\n\nRefer to the standard library\'s `warnings` documentation for more usages.\n\n### Type assertion\n\nAnother way to use the `TJ` objects is to have them check the type of value you are accessing. Consider this case of Bob refusing to play the game properly:\n\n```py\n>>> high_scores = {\'Alice\': 123, \'Bob\': None, \'Charlie\': 456}\n>>> alice_score = high_scores[\'Alice\']\n>>> bob_score = high_scores[\'Bob\']\n>>> charlie_score = high_scores[\'Charlie\']\n>>> print(max(alice_score, bob_score, charlie_score))\nTraceback (most recent call last):\n  File "<stdin>", line 1, in <module>\nTypeError: \'>\' not supported between instances of \'NoneType\' and \'int\'\n```\n\n`TJ` can come to the rescue, using the `.number` assertion to create a copy of itself that checks that it _actually_ contains a number! Like with access problems, any issue results in a warning, and the `TJ` containing the default value of the appropriate type (in this case, `0`).\n\n```py\n>>> high_scores = TJ({\'Alice\': 123, \'Bob\': None, \'Charlie\': 456})\n>>> alice_score = high_scores[\'Alice\'].number.value\n>>> bob_score = high_scores[\'Bob\'].number.value\n<stdin>:1: TypeMismatchWarning: Cannot cast to int|float at path `.Bob`\n>>> charlie_score = high_scores[\'Charlie\'].number.value\n>>> print(max(alice_score, bob_score, charlie_score))\n456\n```\n\nNo more error! But what if you wanted `None` to actually be a valid value? That\'s cool, you can use `.number_or_null` instead, and `None` will be considered a "technically valid" value for the number.\n\nThe type assertions supported this way correspond to the different types in JSON, plus their "nullable" versions. They map to the following Python type hints:\n\n* `bool` &rarr; `bool`\n* `bool_or_null` &rarr; `Optional[bool]`\n* `string` &rarr; `str`\n* `string_or_null` &rarr; `Optional[str]`\n* `number` &rarr; `int | float`  \n* `number_or_null` &rarr; `Optional[int | float]`\n* `array` &rarr; `List[...]`\n* `array_or_null` &rarr; `Optional[List[...]]`\n* `object` &rarr; `Dict[str, ...]`\n* `object_or_null` &rarr; `Optional[Dict[str, ...]]` \n\n> Note: JSON does not distinguish between integer and floating point numbers, so `tjson` does not either. If your code cares, you need to handle it separately.\n\n> Note: The only valid object keys in JSON are strings. Integers are not valid object keys, and need to be wrapped in strings to be used as such.\n\n### Type hints for your IDE\n\n`tjson` is built using type hints on all the right places. Your IDE should be able to pick these up in order to provide you a rich type checking experience.\n\nFor example, using PyLance in Visual Studio Code, I can see that my string operation is potentially unsafe since I might be adding `None` to a string:\n\n![](./type_hint_example.png)\n\n### More...?\n\n`tjson` has good test coverage. Check its tests (`test_*.py`) to see all the features.\n\n## License: MIT\n\nLike `tjson`? Do what you like with it, just give credit.\n\nDon\'t like it? Think it\'s pointless or trivial? That\'s probably fair, but it\'s here anyway.\n\n## Enjoy, and happy coding!',
    'author': 'Filip Sufitchi',
    'author_email': 'fsufitchi@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/fsufitch/tjson',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
